package com.gic.demo.project.web.interceptor;

import com.alibaba.fastjson.JSONObject;
import com.gic.commons.interceptor.HeaderParamInterceptor;
import com.gic.commons.util.StringUtil;
import com.gic.dubbo.entity.ProviderLocalTag;
import com.gic.dubbo.entity.TraceNode;
import org.apache.dubbo.rpc.RpcContext;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Map;
import java.util.UUID;

/**
 * Created by paste on 2018/8/2 15:24
 * <p>
 * TODO: class description
 */
public class HeaderTagInterceptor extends HandlerInterceptorAdapter{
    private static final String HEADER_TAG = "X-Request-Id".toLowerCase();
    static Logger logger = LogManager.getLogger(HeaderTagInterceptor.class);
    static Logger paramLogger = LogManager.getLogger(HeaderParamInterceptor.class);

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
//        return super.preHandle(request, response, handler);


        RpcContext rpcContext = RpcContext.getContext();
        String headerTag = request.getHeader(HEADER_TAG);
        ProviderLocalTag localTag = ProviderLocalTag.tag.get();
        localTag.traceNode = new TraceNode();
        localTag.traceNode.setTimeCost(System.currentTimeMillis());
        if (StringUtil.isNotNullStr(headerTag, true)) {
            rpcContext.getAttachments().put(ProviderLocalTag.TAG_KEY, headerTag);
            localTag.traceId = headerTag;
        } else {
            String traceId = UUID.randomUUID().toString();
            rpcContext.getAttachments().put(ProviderLocalTag.TAG_KEY, traceId);
            localTag.traceId = traceId;
        }


        return true;
    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler, Exception ex) throws Exception {
        ProviderLocalTag localTag = ProviderLocalTag.tag.get();
        response.setHeader(HEADER_TAG, ProviderLocalTag.tag.get().traceId);
        super.afterCompletion(request, response, handler, ex);

        // consumer调用异常
        if (localTag.currentTraceNode != null) {
            localTag.currentTraceNode.setTimeCost(System.currentTimeMillis()- localTag.currentTraceTime);
            localTag.traceNode.addChildNode(localTag.currentTraceNode);

            localTag.currentTraceNode = null;
        }

        if (localTag.traceNode.getChildNodes().size() > 0) {
            localTag.traceNode.setTimeCost(System.currentTimeMillis() - localTag.traceNode.getTimeCost());
            logger.info(JSONObject.toJSONString(localTag.traceNode));
        }
        if (handler instanceof HandlerMethod) {
            Map<String, String[]> parameterMap = request.getParameterMap();
//            Set<Map.Entry<String, String[]>> entries = parameterMap.entrySet();
            JSONObject object = new JSONObject();
            if (parameterMap.size() > 0) {
                object = JSONObject.parseObject(JSONObject.toJSONString(parameterMap));
            }


            if (request.getAttribute("clerkId") != null) {
                object.put("clerkId", request.getAttribute("clerkId"));
                object.put("enterpriseId", request.getAttribute("enterpriseId"));
            }
            paramLogger.info(object.toJSONString());
        }
    }
}
